/**
 *
 * Agora Real Time Engagement
 * Copyright (c) 2024 Agora IO. All rights reserved.
 *
 */

#import <Foundation/Foundation.h>

#import "AgoraRteEnumerates.h"

@class AgoraRteError;
@class AgoraRte;
@class AgoraRtePlayerCustomSourceProvider;
@class AgoraRteStream;
@class AgoraRteCanvas;
@class AgoraRtePlayerObserver;

/**
 * The PlayerInitialConfig class is used to initialize the Player object.
 * @since v4.4.0
 * @technical preview
 */
__attribute__((visibility("default"))) @interface AgoraRtePlayerInitialConfig : NSObject

-(instancetype _Nonnull)init;

@end

/**
 * The PlayerConfig class is used to manage the config.
 * @since v4.4.0
 * @technical preview
 */
__attribute__((visibility("default"))) @interface AgoraRtePlayerConfig : NSObject

-(instancetype _Nonnull)init;

- (void)setAutoPlay:(BOOL)autoPlay error:(AgoraRteError * _Nullable)error;
- (BOOL)autoPlay:(AgoraRteError * _Nullable)error;

- (void)setPlaybackSpeed:(int32_t)speed error:(AgoraRteError * _Nullable)error;
- (int32_t)playbackSpeed:(AgoraRteError * _Nullable)error;

- (void)setPlayoutAudioTrackIdx:(int)idx error:(AgoraRteError * _Nullable)error;
- (int)playoutAudioTrackIdx:(AgoraRteError * _Nullable)error;

- (void)setPublishAudioTrackIdx:(int)idx error:(AgoraRteError * _Nullable)error;
- (int)publishAudioTrackIdx:(AgoraRteError * _Nullable)error;

- (void)setAudioTrackIdx:(int)idx error:(AgoraRteError * _Nullable)error;
- (int)audioTrackIdx:(AgoraRteError * _Nullable)error;

- (void)setSubtitleTrackIdx:(int)idx error:(AgoraRteError * _Nullable)error;
- (int)subtitleTrackIdx:(AgoraRteError * _Nullable)error;

- (void)setExternalSubtitleTrackIdx:(int)idx error:(AgoraRteError * _Nullable)error;
- (int)externalSubtitleTrackIdx:(AgoraRteError * _Nullable)error;

- (void)setAudioPitch:(int32_t)pitch error:(AgoraRteError * _Nullable)error;
- (int32_t)audioPitch:(AgoraRteError * _Nullable)error;

- (void)setPlayoutVolume:(int32_t)volume error:(AgoraRteError * _Nullable)error;
- (int32_t)playoutVolume:(AgoraRteError * _Nullable)error;

- (void)setAudioPlaybackDelay:(int32_t)delay error:(AgoraRteError * _Nullable)error;
- (int32_t)audioPlaybackDelay:(AgoraRteError * _Nullable)error;

- (void)setAudioDualMonoMode:(int)mode error:(AgoraRteError * _Nullable)error;
- (int)audioDualMonoMode:(AgoraRteError * _Nullable)error;

- (void)setPublishVolume:(int32_t)volume error:(AgoraRteError * _Nullable)error;
- (int32_t)publishVolume:(AgoraRteError * _Nullable)error;

- (void)setLoopCount:(int32_t)count error:(AgoraRteError * _Nullable)error;
- (int32_t)loopCount:(AgoraRteError * _Nullable)error;

- (void)setJsonParameter:(NSString * _Nonnull)jsonParameter error:(AgoraRteError * _Nullable)error;
- (NSString * _Nullable)jsonParameter:(AgoraRteError * _Nullable)error;

- (void)setAbrSubscriptionLayer:(AgoraRteAbrSubscriptionLayer)layer error:(AgoraRteError * _Nullable)error;
- (AgoraRteAbrSubscriptionLayer)abrSubscriptionLayer:(AgoraRteError * _Nullable)error;

- (void)setAbrFallbackLayer:(AgoraRteAbrFallbackLayer)layer error:(AgoraRteError * _Nullable)error;
- (AgoraRteAbrFallbackLayer)abrFallbackLayer:(AgoraRteError * _Nullable)error;

@end

/**
 * The PlayerStats class is used to manage the stats.
 * @since v4.4.0
 * @technical preview
 */
__attribute__((visibility("default"))) @interface AgoraRtePlayerStats : NSObject

- (instancetype _Nonnull)init;

- (int)videoDecodeFrameRate;
- (int)videoRenderFrameRate;
- (int)videoBitrate;
- (int)audioBitrate;

@end

/**
 * The PlayerInfo class is used to manage the info.
 * @since v4.4.0
 * @technical preview
 */
__attribute__((visibility("default"))) @interface AgoraRtePlayerInfo : NSObject
- (instancetype _Nonnull)init;

- (int)state;
- (long)duration;
- (int)streamCount;
- (BOOL)hasAudio;
- (BOOL)hasVideo;
- (BOOL)isAudioMuted;
- (BOOL)isVideoMuted;
- (int)videoHeight;
- (int)videoWidth;
- (AgoraRteAbrSubscriptionLayer)abrSubscriptionLayer;
- (int)audioSampleRate;
- (int)audioChannels;
- (int)audioBitsPerSample;

@end


/**
 * The Player class is used to manage the player.
 * @since v4.4.0
 * @technical preview
 */
__attribute__((visibility("default"))) @interface AgoraRtePlayer : NSObject

- (instancetype _Nonnull)initWithRte:(AgoraRte * _Nonnull)rte initialConfig:(AgoraRtePlayerInitialConfig * _Nullable)config;

- (BOOL)preloadWithUrl:(NSString * _Nonnull)url error:(AgoraRteError * _Nullable)error;

- (void)openWithUrl:(NSString * _Nonnull)url startTime:(uint64_t)startTime cb:(void (^_Nullable)(AgoraRteError* _Nullable err))cb;

- (void)openWithCustomSourceProvider:(AgoraRtePlayerCustomSourceProvider * _Nonnull)provider startTime:(uint64_t)startTime cb:(void (^_Nullable)(AgoraRteError* _Nullable err))cb;

- (void)openWithStream:(AgoraRteStream * _Nonnull)stream cb:(void (^_Nullable)(AgoraRteError* _Nullable err))cb;

- (void)getStats:(void (^_Nonnull)(AgoraRtePlayerStats* _Nonnull stats, AgoraRteError* _Nullable err))cb ;

- (BOOL)setCanvas:(AgoraRteCanvas *_Nonnull)canvas error:(AgoraRteError * _Nullable)error;

- (BOOL)play:(AgoraRteError * _Nullable)error;

- (BOOL)stop:(AgoraRteError * _Nullable)error;

- (BOOL)pause:(AgoraRteError * _Nullable)error;

- (BOOL)seek:(uint64_t)newTime error:(AgoraRteError * _Nullable)error;

- (BOOL)muteAudio:(BOOL)mute error:(AgoraRteError * _Nullable)error;

- (BOOL)muteVideo:(BOOL)mute error:(AgoraRteError * _Nullable)error;

- (uint64_t)getPosition:(AgoraRteError * _Nullable)error;

- (BOOL)getInfo:(AgoraRtePlayerInfo * _Nonnull)info error:(AgoraRteError * _Nullable)error;

- (BOOL)getConfigs:(AgoraRtePlayerConfig * _Nonnull)config error:(AgoraRteError * _Nullable)error;

- (BOOL)setConfigs:(AgoraRtePlayerConfig * _Nonnull)config error:(AgoraRteError * _Nullable)error;

- (BOOL)registerObserver:(AgoraRtePlayerObserver *_Nonnull)observer error:(AgoraRteError * _Nullable)error;

- (BOOL)unregisterObserver:(AgoraRtePlayerObserver * _Nullable)observer error:(AgoraRteError * _Nullable)error;

@end
